import redis from 'redis';
// 内存存储
import nodeCache from 'node-cache';
import util from 'util';
import consola from 'consola/dist/consola';
import config from '@config';

// 是否使用redis
let isUseRedis: boolean = true
// 内存存储实例化
const cache = new nodeCache()

// 创建redis连接
const client = redis.createClient({
  host: config.redis.host,
  port: config.redis.port,
  password: config.redis.password,
  auth_pass: config.redis.password,
  retry_strategy: function(options) {
    if (options.error && options.error.code === "ECONNREFUSED") {
      // redis连接失败时使用内存存储
      isUseRedis = false
      // End reconnecting on a specific error and flush all commands with
      // a individual error
      return new Error("The server refused the connection");
    }
    if (options.total_retry_time > 1000 * 60 * 60) {
      // End reconnecting after a specific timeout and flush all commands
      // with a individual error
      return new Error("Retry time exhausted");
    }
    if (options.attempt > 10) {
      // End reconnecting with built in error
      return;
    }
    // reconnect after
    return Math.min(options.attempt * 1000, 3000);
  }
});
client.on('error', err => {
  consola.error(err);
});
client.on('ready', () => {
  consola.success('redis is ready');
});
client.on('connect', () => {
  consola.success('redis is connecting');
  isUseRedis = true
});
client.on('reconnecting', () => {
  consola.success('redis is reconnecting...');
});
client.on('end', () => {
  consola.success('redis is closed');
});

// redis查询和设置await/async化
const promisify = util.promisify;
const getClient = (key) => promisify(client.get).bind(client)(key);
const setClient = (key, value) => promisify(client.set).bind(client)(key, value);
const setexClient = (key, ttl, value) => promisify(client.setex).bind(client)(key, ttl, value);

// 内存存储promise化，为了和redis使用方法保持一致
const getCache = (key: string): Promise<string|undefined> => {
  console.log(key);
  return new Promise(resolve => {
    resolve(cache.get(key))
  })
}
const setCache = (key: string, ttl: number = 0, value: string): Promise<boolean> => {
  return new Promise((resolve, reject) => {
    const success = cache.set(key, value, ttl)
    if (success) {
      resolve(success)
    } else {
      reject(success)
    }
  })
}
const setexCache = (key: string, ttl: number, value: string, ) => setCache(key, ttl, value)

// 初始化测试
// client.setex('test', 30, '111111');
// send_sms_verify_code_sms_13500001111

export default () => {
  return async (ctx, next) => {
    consola.info('redis is available:', isUseRedis);
    if (isUseRedis) { // 如果redis连接失败，采用内存
      ctx.$redis = { set: setClient, get: getClient, setex: setexClient };
    } else {
      ctx.$redis = {
        set: setCache,
        get: getCache,
        setex: setexCache
      }
    }
    await next();
  };
};
